@@ -1,20 +1,19 @@
-"""Tests for Neighboring Relations"""
-# pylint: disable=pointless-string-statement, no-self-use, no-member
+"""Tests for neighboring relations."""
+
+# SPDX-License-Identifier: Apache-2.0
+# Copyright Tumult Labs 2025
+
 from cmath import inf, pi
+from typing import Dict
 
 import pandas as pd
 import pytest
 import sympy as sp
-
-from tmlt.analytics._neighboring_relation_visitor import NeighboringRelationCoreVisitor
-from tmlt.analytics._neighboring_relations import (
-    AddRemoveRows,
-    AddRemoveRowsAcrossGroups,
-    Conjunction,
-)
+from pyspark.sql import DataFrame
 from tmlt.core.domains.collections import DictDomain
 from tmlt.core.domains.spark_domains import SparkDataFrameDomain
 from tmlt.core.measures import PureDP, RhoZCDP
+from tmlt.core.metrics import AddRemoveKeys as CoreAddRemoveKeys
 from tmlt.core.metrics import (
     DictMetric,
     IfGroupedBy,
@@ -23,6 +22,16 @@ from tmlt.core.metrics import (
     SymmetricDifference,
 )
 from tmlt.core.utils.exact_number import ExactNumber
+from typeguard import TypeCheckError
+
+from tmlt.analytics._neighboring_relation import (
+    AddRemoveKeys,
+    AddRemoveRows,
+    AddRemoveRowsAcrossGroups,
+    Conjunction,
+)
+from tmlt.analytics._neighboring_relation_visitor import NeighboringRelationCoreVisitor
+from tmlt.analytics._table_identifier import NamedTable, TableCollection
 
 
 @pytest.fixture(name="test_data", scope="class")
@@ -69,6 +78,12 @@ def setup_test_data(request, spark) -> None:
 class TestNeighboringRelations:
     """Tests for the AddRemoveRows NeighboringRelation."""
 
+    table1: DataFrame
+    table2: DataFrame
+    table3: DataFrame
+    testdfsdict: Dict[str, DataFrame]
+    testdfsdictgrouping: Dict[str, DataFrame]
+
     def test_add_remove_rows_validation(self):
         """Test that validate works as expected for AddRemoveRows."""
         valid_dict = {"table1": self.table1}
@@ -80,9 +95,9 @@ class TestNeighboringRelations:
         with pytest.raises(ValueError):
             AddRemoveRows("table1", n=1).validate_input(self.testdfsdict)
         # table's value is of wrong type
-        with pytest.raises(TypeError):
+        with pytest.raises(TypeCheckError):
             AddRemoveRows("table1", n=1).validate_input(
-                {"table1": ["a", "random", "list"]}
+                {"table1": ["a", "random", "list"]}  # type: ignore
             )
 
     def test_add_remove_rows_accept(self):
@@ -123,9 +138,9 @@ class TestNeighboringRelations:
                 self.testdfsdictgrouping
             )
         # table's value is not a DataFrame
-        with pytest.raises(TypeError):
+        with pytest.raises(TypeCheckError):
             AddRemoveRowsAcrossGroups("table1", "B", 1, 1).validate_input(
-                {"table1": ["a", "random", "list"]}
+                {"table1": ["a", "random", "list"]}  # type: ignore
             )
         # table contains values not supported in grouping operations
         with pytest.raises(ValueError):
@@ -158,6 +173,70 @@ class TestNeighboringRelations:
             self.table1,
         )
 
+    #### Tests for AddRemoveKeys ####
+    def test_add_remove_keys_post_init(self) -> None:
+        """Test post init for AddRemoveKeys."""
+        key_dict = {"table1": "A"}
+        identifier = "item_id"
+        with pytest.raises(ValueError):
+            AddRemoveKeys(identifier, key_dict, max_keys=0)
+        with pytest.raises(ValueError):
+            AddRemoveKeys(identifier, key_dict, max_keys=-1)
+
+    def test_add_remove_keys_validation(self) -> None:
+        """Test that validate works as expected for AddRemoveKeys."""
+        key_dict = {"table1": "A", "table2": "A"}
+        valid_dict = {"table1": self.table1, "table2": self.table2}
+        identifier = "item_id"
+        assert AddRemoveKeys(identifier, key_dict).validate_input(valid_dict)
+        assert AddRemoveKeys(identifier, key_dict).validate_input(valid_dict)
+        # Input dict has too few elements
+        with pytest.raises(ValueError):
+            AddRemoveKeys(identifier, key_dict).validate_input({"table1": self.table1})
+        # Table is missing the requested column
+        with pytest.raises(ValueError):
+            AddRemoveKeys(
+                identifier, {"table1": "column_that_does_not_exist"}
+            ).validate_input({"table1": self.table1})
+        # Column has the wrong type
+        with pytest.raises(ValueError):
+            AddRemoveKeys("p_id", {"table3": "A"}).validate_input(
+                {"table3": self.table3}
+            )
+        # Column types don't match
+        with pytest.raises(ValueError):
+            AddRemoveKeys("p_id", {"table1": "A", "table2": "B"}).validate_input(
+                valid_dict
+            )
+
+    def test_add_remove_keys_accept(self) -> None:
+        """Test that accept works as expected for AddRemoveKeys."""
+        pure_visitor = NeighboringRelationCoreVisitor(
+            self.testdfsdict, output_measure=PureDP()
+        )
+        relation = AddRemoveKeys("p_id", {"table1": "B", "table2": "X"}, max_keys=3)
+        expected = NeighboringRelationCoreVisitor.Output(
+            DictDomain(
+                {
+                    NamedTable("table1"): SparkDataFrameDomain.from_spark_schema(
+                        self.table1.schema
+                    ),
+                    NamedTable("table2"): SparkDataFrameDomain.from_spark_schema(
+                        self.table2.schema
+                    ),
+                }
+            ),
+            CoreAddRemoveKeys({NamedTable("table1"): "B", NamedTable("table2"): "X"}),
+            ExactNumber(3),
+            {NamedTable("table1"): self.table1, NamedTable("table2"): self.table2},
+        )
+
+        assert relation.accept(pure_visitor) == expected
+        rho_visitor = NeighboringRelationCoreVisitor(
+            self.testdfsdict, output_measure=RhoZCDP()
+        )
+        assert relation.accept(rho_visitor) == expected
+
     #### Tests for Conjunction ####
 
     def test_conjunction_initialization(self):
@@ -185,31 +264,70 @@ class TestNeighboringRelations:
         expected_add_remove_rows_groups_result = AddRemoveRowsAcrossGroups(
             "table2", "A", 2, 1
         ).accept(visitor)
-        assert Conjunction(
+
+        output = Conjunction(
             AddRemoveRows("table1", 1), AddRemoveRowsAcrossGroups("table2", "A", 2, 1)
-        ).accept(visitor) == (
+        ).accept(visitor)
+        expected_output = NeighboringRelationCoreVisitor.Output(
             DictDomain(
                 {
-                    "table1": expected_add_remove_rows_result[0],
-                    "table2": expected_add_remove_rows_groups_result[0],
+                    NamedTable("table1"): expected_add_remove_rows_result[0],
+                    NamedTable("table2"): expected_add_remove_rows_groups_result[0],
                 }
             ),
             DictMetric(
                 {
-                    "table1": expected_add_remove_rows_result[1],
-                    "table2": expected_add_remove_rows_groups_result[1],
+                    NamedTable("table1"): expected_add_remove_rows_result[1],
+                    NamedTable("table2"): expected_add_remove_rows_groups_result[1],
                 }
             ),
             {
-                "table1": expected_add_remove_rows_result[2],
-                "table2": expected_add_remove_rows_groups_result[2],
+                NamedTable("table1"): expected_add_remove_rows_result[2],
+                NamedTable("table2"): expected_add_remove_rows_groups_result[2],
             },
             {
-                "table1": expected_add_remove_rows_result[3],
-                "table2": expected_add_remove_rows_groups_result[3],
+                NamedTable("table1"): expected_add_remove_rows_result[3],
+                NamedTable("table2"): expected_add_remove_rows_groups_result[3],
             },
         )
 
+        assert output == expected_output
+
+    def test_conjunction_accept_with_add_remove_keys(self) -> None:
+        """Test Conjunction.accept when Conjunction contains AddRemoveKeys."""
+        dfs_dict = {"table1": self.table1, "table2": self.table2, "table3": self.table3}
+        visitor = NeighboringRelationCoreVisitor(dfs_dict, PureDP())
+        add_remove_keys = AddRemoveKeys("p_id", {"table1": "B", "table2": "X"})
+        expected_add_remove_keys_result = add_remove_keys.accept(visitor)
+        add_remove_rows = AddRemoveRows("table3", 1)
+        expected_add_remove_rows_result = add_remove_rows.accept(visitor)
+
+        output = Conjunction(add_remove_keys, add_remove_rows).accept(visitor)
+
+        expected_output = NeighboringRelationCoreVisitor.Output(
+            DictDomain(
+                {
+                    TableCollection("p_id"): expected_add_remove_keys_result[0],
+                    NamedTable("table3"): expected_add_remove_rows_result[0],
+                }
+            ),
+            DictMetric(
+                {
+                    TableCollection("p_id"): expected_add_remove_keys_result[1],
+                    NamedTable("table3"): expected_add_remove_rows_result[1],
+                }
+            ),
+            {
+                TableCollection("p_id"): expected_add_remove_keys_result[2],
+                NamedTable("table3"): expected_add_remove_rows_result[2],
+            },
+            {
+                TableCollection("p_id"): expected_add_remove_keys_result[3],
+                NamedTable("table3"): expected_add_remove_rows_result[3],
+            },
+        )
+        assert output == expected_output
+
     def test_conjunction_validation(self):
         """Tests that validate_input works as expected for Conjunction."""
         assert Conjunction(
